# -*- encoding: utf-8 -*-

##############################################################################
#
# Copyright (c) 2010 E.Samyn - info@open-shift.com
# Author: E.Samyn
#
# This file is part of the hr_contract_extension module
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsability of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# garantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; 
# If not, see <http://www.gnu.org/licenses/> or  write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################

from osv import fields, osv
from mx import DateTime

import datetime

# ponctual payments (bonus, commission,...)
class contract_ponctual_payment(osv.osv):
	_name = 'contract.ponctual.payment'
	_description = 'ponctual payment object'
	_columns = {
		'name' : fields.char('Label', size=60, required=True),
		'start_date' : fields.date('Period start date'),
		'end_date' : fields.date('Period end date'),
		'payment_date' : fields.date('Payment date'),
		'amount' : fields.float('Amount', digits=(16,2)),
		'contract_id' : fields.many2one('hr.contract','Contract'),	
	}
contract_ponctual_payment()

# hr_contract extension
class hr_contract(osv.osv):
	_name = 'hr.contract'
	_description = 'contract extension'
	_inherit = "hr.contract"
	_columns = {
		'parent_id' : fields.many2one('hr.contract','Parent contract',help='For sub contracts'),
		'working_place' : fields.char('Contract location', size=60),
		'trial_extension_end_date' : fields.date('Trial extension end date'),
		'work_permit_type' : fields.selection([('student','Student'),
 			                               ('employee','Employee'),
										   ('other','Other')],'Work permit type'),
		'work_permit_validity' : fields.date('Work permit validity'),
		'current_job_level' : fields.many2one('hr.job.level','Current job level'), # IMPROVEMENT TO DO : set fields job_id & current_job_level of the employee form readonly and create a function that updates those fields with the last contract details
		'payments_ids': fields.one2many('contract.ponctual.payment', 'contract_id', 'Ponctual payments',help='For bonus, commissions,...'),
		'seniority_rate' : fields.float('Seniority rate', digits=(6,2), help='To calculate internal seniority (0 to 1)'),
	}

hr_contract()

class hr_employee(osv.osv):
    _name = "hr.employee"
    _description = "Employee seniority"
    _inherit = "hr.employee"

#--- Method to calculate the total seniority	
    def _get_total_seniority(self,cr,uid,ids,field_name,arg,context):
		res = {}
		employees = self.browse(cr, uid, ids)
		for employee in employees:
			res[employee.id] = 0.0
			if employee.internal_seniority:
				total = employee.external_seniority + employee.internal_seniority
				res[employee.id] = total
			else:
				res[employee.id] = employee.external_seniority
		return res	

#--- Method to calculate the internal seniority 
    def get_internal_seniority(self,cr,uid,ids,*args):
		start_date = datetime.date.today()
		end_date = datetime.date.today() # if the last contract has no en date, en date = today
		internal_seniority = 0.0
		internal_year_seniority = 0.0
		internal_month_seniority = 0.0
		# Get contracts for employee
		contract_pool = self.pool.get('hr.contract')
		contract_ids = contract_pool.search(cr,uid,[('employee_id','=',ids[0])],order='date_start desc') # contracts from today to first based on start date
		contracts = contract_pool.browse(cr, uid, contract_ids)
		# Get seniority for each contract
		for contract in contracts:
			seniority_rate = 1 # default seniority
			start_date = DateTime.strptime(contract.date_start,'%Y-%m-%d')
			if contract.seniority_rate:
				seniority_rate = contract.seniority_rate 
			if contract.date_end:
				end_date = DateTime.strptime(contract.date_end,'%Y-%m-%d')
			internal_year_seniority += (end_date.year - start_date.year)*seniority_rate*1.0 # *1.0 to get a float
			internal_month_seniority += (end_date.month - start_date.month + 1)*seniority_rate*1.0	# +1 : a started month is counted as a full month
			end_date = start_date # if previous contract (in time scale) has no end date, its supposed end date is the current contract start date
		# set seniority in years
			internal_seniority = internal_year_seniority + internal_month_seniority/12 + internal_month_seniority//12
		# Update internal seniority field
		self.write(cr,uid,ids,{'internal_seniority':internal_seniority})
		return True
		
    _columns = {
        'external_seniority': fields.float('External seniority', digits=(6,2), help='Seniority gained outside the compagny'),
        'internal_seniority': fields.float('Internal seniority', digits=(6,2), help="""Seniority gained inside the compagny
Calculated from employee contracts in start date desc order: sum((end_date-start_date)*seniority_rate)
A started month is counted as full.
Need to refresh the form to display new value."""),
		'total_seniority': fields.function(_get_total_seniority, method=True, string='Total seniority', type='float', help='External+internal'),
        }

hr_employee()